@@ -2,7 +2,6 @@ import argparse
 import atexit
 import json
 import logging
-import os
 import shutil
 import ssl
 import subprocess
@@ -25,7 +24,7 @@ AVClientRequest = AVClientResponse = Dict[str, Any]
 
 class SmartDetectObjectType(Enum):
     PERSON = "person"
-    CAR = "car"
+    VEHICLE = "vehicle"
 
 
 class UnifiCamBase(metaclass=ABCMeta):
@@ -36,10 +35,11 @@ class UnifiCamBase(metaclass=ABCMeta):
         self._msg_id: int = 0
         self._init_time: float = time.time()
         self._streams: Dict[str, str] = {}
-        self._motion_snapshot = None
+        self._motion_snapshot: Optional[Path] = None
         self._motion_event_id: int = 0
         self._motion_event_ts: Optional[float] = None
-        self._ffmpeg_handles = {}
+        self._motion_object_type: Optional[SmartDetectObjectType] = None
+        self._ffmpeg_handles: Dict[str, subprocess.Popen] = {}
 
         # Set up ssl context for requests
         self._ssl_context = ssl.create_default_context()
@@ -54,7 +54,7 @@ class UnifiCamBase(metaclass=ABCMeta):
         parser.add_argument(
             "--ffmpeg-args",
             "-f",
-            default="-c:v copy -ar 32000 -ac 2 -codec:a aac -b:a 32k",
+            default="-c:v copy -ar 32000 -ac 1 -codec:a aac -b:a 32k",
             help="Transcoding args for `ffmpeg -i <src> <args> <dst>`",
         )
         parser.add_argument(
@@ -71,6 +71,7 @@ class UnifiCamBase(metaclass=ABCMeta):
             try:
                 msg = await ws.recv()
             except websockets.exceptions.ConnectionClosedError:
+                self.logger.info(f"Connection to {self.args.host} was closed.")
                 raise RetryableError()
 
             if msg is not None:
@@ -82,10 +83,10 @@ class UnifiCamBase(metaclass=ABCMeta):
     async def run(self) -> None:
         pass
 
-    def get_video_settings(self) -> Dict[str, Any]:
+    async def get_video_settings(self) -> Dict[str, Any]:
         return {}
 
-    def change_video_settings(self, options) -> None:
+    async def change_video_settings(self, options) -> None:
         pass
 
     @abstractmethod
@@ -93,12 +94,18 @@ class UnifiCamBase(metaclass=ABCMeta):
         raise NotImplementedError("You need to write this!")
 
     @abstractmethod
-    def get_stream_source(self, stream_index: str) -> str:
+    async def get_stream_source(self, stream_index: str) -> str:
         raise NotImplementedError("You need to write this!")
 
-    def get_feature_flags(self) -> Dict[str, Any]:
+    def get_extra_ffmpeg_args(self) -> str:
+        return self.args.ffmpeg_args
+
+    async def get_feature_flags(self) -> Dict[str, Any]:
         return {
             "mic": True,
+            "aec": [],
+            "videoMode": ["default"],
+            "motionDetect": ["enhanced"],
         }
 
     # API for subclasses
@@ -130,7 +137,12 @@ class UnifiCamBase(metaclass=ABCMeta):
                     }
                 )
 
-            self.logger.info(f"Triggering motion start (idx: {self._motion_event_id})")
+            self.logger.info(
+                f"Triggering motion start (idx: {self._motion_event_id})"
+                + f" for {object_type.value}"
+                if object_type
+                else ""
+            )
             await self.send(
                 self.gen_response(
                     "EventSmartDetect" if object_type else "EventAnalytics",
@@ -138,14 +150,20 @@ class UnifiCamBase(metaclass=ABCMeta):
                 ),
             )
             self._motion_event_ts = time.time()
-            self._motion_snapshot = shutil.copyfile(
-                await self.get_snapshot(), tempfile.NamedTemporaryFile(delete=True).name
-            )
+            self._motion_object_type = object_type
 
-    async def trigger_motion_stop(
-        self, object_type: Optional[SmartDetectObjectType] = None
-    ) -> None:
+            # Capture snapshot at beginning of motion event for thumbnail
+            motion_snapshot_path: str = tempfile.NamedTemporaryFile(delete=False).name
+            try:
+                shutil.copyfile(await self.get_snapshot(), motion_snapshot_path)
+                self.logger.debug(f"Captured motion snapshot to {motion_snapshot_path}")
+                self._motion_snapshot = Path(motion_snapshot_path)
+            except FileNotFoundError:
+                pass
+
+    async def trigger_motion_stop(self) -> None:
         motion_start_ts = self._motion_event_ts
+        motion_object_type = self._motion_object_type
         if motion_start_ts:
             payload: Dict[str, Any] = {
                 "clockBestMonotonic": int(self.get_uptime()),
@@ -161,24 +179,30 @@ class UnifiCamBase(metaclass=ABCMeta):
                 "motionHeatmap": "heatmap.png",
                 "motionSnapshot": "motionsnap.jpg",
             }
-            if object_type:
+            if motion_object_type:
                 payload.update(
                     {
-                        "objectTypes": [object_type.value],
+                        "objectTypes": [motion_object_type.value],
                         "edgeType": "leave",
                         "zonesStatus": {"0": 48},
                         "smartDetectSnapshot": "motionsnap.jpg",
                     }
                 )
-            self.logger.info(f"Triggering motion stop (idx: {self._motion_event_id})")
+            self.logger.info(
+                f"Triggering motion stop (idx: {self._motion_event_id})"
+                + f" for {motion_object_type.value}"
+                if motion_object_type
+                else ""
+            )
             await self.send(
                 self.gen_response(
-                    "EventSmartDetect" if object_type else "EventAnalytics",
+                    "EventSmartDetect" if motion_object_type else "EventAnalytics",
                     payload=payload,
                 ),
             )
             self._motion_event_id += 1
             self._motion_event_ts = None
+            self._motion_object_type = None
 
     def update_motion_snapshot(self, path: Path) -> None:
         self._motion_snapshot = path
@@ -220,8 +244,8 @@ class UnifiCamBase(metaclass=ABCMeta):
                     "semver": "v4.4.8",
                     "totalLoad": 0.5474,
                     "upgradeTimeoutSec": 150,
-                    "uptime": self.get_uptime(),
-                    "features": self.get_feature_flags(),
+                    "uptime": int(self.get_uptime()),
+                    "features": await self.get_feature_flags(),
                 },
             ),
         )
@@ -232,7 +256,7 @@ class UnifiCamBase(metaclass=ABCMeta):
             msg["messageId"],
             {
                 "authToken": self.args.token,
-                "features": self.get_feature_flags(),
+                "features": await self.get_feature_flags(),
             },
         )
 
@@ -296,7 +320,7 @@ class UnifiCamBase(metaclass=ABCMeta):
             "wdr": 1,
             "zoomPosition": 0,
         }
-        payload.update(self.get_video_settings())
+        payload.update(await self.get_video_settings())
         return self.gen_response(
             "ResetIspSettings",
             msg["messageId"],
@@ -325,7 +349,7 @@ class UnifiCamBase(metaclass=ABCMeta):
                                 host, port = urllib.parse.urlparse(
                                     v["avSerializer"]["destinations"][0]
                                 ).netloc.split(":")
-                                self.start_video_stream(
+                                await self.start_video_stream(
                                     k, stream, destination=(host, int(port))
                                 )
                             except ValueError:
@@ -352,6 +376,7 @@ class UnifiCamBase(metaclass=ABCMeta):
                     "enableHrd": False,
                     "hdrMode": 0,
                     "lowDelay": False,
+                    "videoMode": "default",
                     "mjpg": {
                         "avSerializer": {
                             "destinations": [
@@ -412,7 +437,7 @@ class UnifiCamBase(metaclass=ABCMeta):
                         "enabled": True,
                         "fps": 15,
                         "gopModel": 0,
-                        "height": 720,
+                        "height": 1080,
                         "horizontalFlip": False,
                         "isCbr": False,
                         "maxFps": 30,
@@ -446,7 +471,7 @@ class UnifiCamBase(metaclass=ABCMeta):
                             30,
                         ],
                         "verticalFlip": False,
-                        "width": 1280,
+                        "width": 1920,
                     },
                     "video2": {
                         "M": 1,
@@ -472,7 +497,7 @@ class UnifiCamBase(metaclass=ABCMeta):
                         "enabled": True,
                         "fps": 15,
                         "gopModel": 0,
-                        "height": 400,
+                        "height": 720,
                         "horizontalFlip": False,
                         "isCbr": False,
                         "maxFps": 30,
@@ -506,7 +531,7 @@ class UnifiCamBase(metaclass=ABCMeta):
                             30,
                         ],
                         "verticalFlip": False,
-                        "width": 720,
+                        "width": 1280,
                     },
                     "video3": {
                         "M": 1,
@@ -717,9 +742,9 @@ class UnifiCamBase(metaclass=ABCMeta):
         }
 
         if msg["payload"]:
-            self.change_video_settings(msg["payload"])
+            await self.change_video_settings(msg["payload"])
 
-        payload.update(self.get_video_settings())
+        payload.update(await self.get_video_settings())
         return self.gen_response("ChangeIspSettings", msg["messageId"], payload)
 
     async def process_analytics_settings(
@@ -738,7 +763,7 @@ class UnifiCamBase(metaclass=ABCMeta):
         else:
             path = await self.get_snapshot()
 
-        if os.path.isfile(path):
+        if path and path.exists():
             async with aiohttp.ClientSession() as session:
                 files = {"payload": open(path, "rb")}
                 files.update(msg["payload"].get("formFields", {}))
@@ -861,27 +886,50 @@ class UnifiCamBase(metaclass=ABCMeta):
 
         return False
 
-    def start_video_stream(
+    def get_base_ffmpeg_args(self) -> str:
+        base_args = [
+            "-avoid_negative_ts",
+            "make_zero",
+            "-fflags",
+            "+genpts+discardcorrupt",
+        ]
+
+        try:
+            output = subprocess.check_output(["ffmpeg", "-h", "full"])
+            if b"stimeout" in output:
+                base_args.append("-stimeout 15000000")
+            else:
+                base_args.append("-timeout 15000000")
+        except subprocess.CalledProcessError:
+            self.logger.exception("Could not check for ffmpeg options")
+
+        return " ".join(base_args)
+
+    async def start_video_stream(
         self, stream_index: str, stream_name: str, destination: Tuple[str, int]
     ):
-        if (
-            stream_index not in self._ffmpeg_handles
-            or self._ffmpeg_handles[stream_index].poll() is not None
-        ):
-            source = self.get_stream_source(stream_index)
+        has_spawned = stream_index in self._ffmpeg_handles
+        is_dead = has_spawned and self._ffmpeg_handles[stream_index].poll() is not None
+
+        if not has_spawned or is_dead:
+            source = await self.get_stream_source(stream_index)
             cmd = (
-                "ffmpeg -nostdin -y -stimeout 15000000"
+                f"ffmpeg -nostdin -loglevel error -y {self.get_base_ffmpeg_args()}"
                 f" -rtsp_transport {self.args.rtsp_transport}"
-                f' -i "{source}" {self.args.ffmpeg_args}'
+                f' -i "{source}" {self.get_extra_ffmpeg_args()}'
                 f" -metadata streamname={stream_name} -f flv -"
                 f" | {sys.executable} -m unifi.clock_sync"
                 f" | nc {destination[0]} {destination[1]}"
             )
+
+            if is_dead:
+                self.logger.warn(f"Previous ffmpeg process for {stream_index} died.")
+
             self.logger.info(
                 f"Spawning ffmpeg for {stream_index} ({stream_name}): {cmd}"
             )
             self._ffmpeg_handles[stream_index] = subprocess.Popen(
-                cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, shell=True
+                cmd, stdout=subprocess.DEVNULL, shell=True
             )
 
     def stop_video_stream(self, stream_index: str):
@@ -891,6 +939,7 @@ class UnifiCamBase(metaclass=ABCMeta):
 
     async def close(self):
         self.logger.info("Cleaning up instance")
+        await self.trigger_motion_stop()
         self.close_streams()
 
     def close_streams(self):
